@@ -175,6 +175,7 @@ const options = {
     // chunkFilename: '[name].bundle.js',
   },
   optimization: {
+    minimize: false, // no point for webext, it loads fast enough anyway
     splitChunks: {
       automaticNameDelimiter: '_', // ugh. default ~ can't be loaded by the browser??
     }
@@ -197,25 +198,6 @@ const options = {
           loader: 'html-loader',
           exclude: /node_modules/
       },
-      {
-          test: /node_modules\/basket.js\/lib\/basket.js/,
-          use: {
-              loader: path.join(__dirname, 'patcher.js'),
-              options: {
-                  patches: [
-                      /* see
-                       * https://github.com/addyosmani/basket.js/issues/174
-                       * https://github.com/addyosmani/basket.js/issues/173
-                       */
-                      {code: /RSVP.Promise/g, newCode: 'Promise'},
-                      {code: 'RSVP.all', newCode: 'Promise.all'},
-                      {code: 'window.basket = {', newCode: 'var basket = window.basket= {'},
-                      // ugh. sometimes it patches it twice somehow??? wtf???
-                      {code: 'var basket = var basket = window.basket= {', newCode: 'var basket = window.basket= {'},
-                  ],
-              }
-          },
-      },
       {
           test: /node_modules\/anchorme\/dist\/browser\/anchorme.js/,
           use: {
