@@ -1,3 +1,5 @@
+from promnesia.common import _is_windows
+
 from common import tmp_popen, promnesia_bin
 
 from pathlib import Path
@@ -5,6 +7,8 @@ import os
 from subprocess import Popen
 import time
 
+import pytest
+
 
 def ox_hugo_data() -> Path:
     p = Path('tests/testdata/ox-hugo/test/site')
@@ -15,6 +19,11 @@ def ox_hugo_data() -> Path:
 
 
 def test_demo() -> None:
+    if _is_windows:
+        # for some reason fails to connect to server..
+        # not sure maybe something with port choice idk
+        pytest.skip("TODO broken on Windows")
+
     import requests
     with tmp_popen(promnesia_bin('demo', '--port', '16789', ox_hugo_data())):
         # FIXME why does it want post??
