@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 from contextlib import contextmanager
+import json
 from pathlib import Path
 from datetime import datetime
 from tempfile import TemporaryDirectory
@@ -19,12 +20,14 @@ if __name__ == '__main__':
 from selenium import webdriver # type: ignore
 from selenium.webdriver.common.keys import Keys
 from selenium.webdriver.common.by import By
+from selenium.webdriver.support.ui import WebDriverWait as Wait
+from selenium.webdriver.support import expected_conditions as EC
 
 
 from common import under_ci, uses_x, has_x
 from integration_test import index_hypothesis, index_local_chrome, index_urls
 from server_test import wserver
-from firefox_helper import open_extension_page
+from browser_helper import open_extension_page, get_cmd_hotkey
 
 from promnesia.logging import LazyLogger
 
@@ -79,21 +82,8 @@ def get_addon_path(kind: str) -> Path:
 
 
 def get_hotkey(driver, cmd: str) -> str:
-    # TODO shit, need to unify this...
-    if driver.name == 'chrome':
-        chrome_profile = Path(driver.capabilities['chrome']['userDataDir'])
-        prefs_file = chrome_profile / 'Default/Preferences'
-        import json
-        prefs = json.loads(prefs_file.read_text())
-        # "commands": {
-        #        "linux:Ctrl+Shift+V": {
-        #            "command_name": "show_dots",
-        #            "extension": "ceedkmkoeooncekjljapnkkjhldddcid",
-        #            "global": false
-        #        }
-        #    },
-        cmd_map = {cmd['command_name']: k.split(':')[-1] for k, cmd in prefs['extensions']['commands'].items()}
-    else:
+    cmd_map = None
+    if driver.name == 'firefox':
         # TODO remove hardcoding somehow...
         # perhaps should be extracted somewhere..
         cmd_map = {
@@ -101,7 +91,7 @@ def get_hotkey(driver, cmd: str) -> str:
             '_execute_browser_action': 'Ctrl+Alt+e',
             'search'                 : 'Ctrl+Alt+h',
         }
-    return cmd_map[cmd].split('+')
+    return get_cmd_hotkey(driver, cmd=cmd, cmd_map=cmd_map)
 
 
 def _get_webdriver(tdir: Path, browser: Browser, extension=True):
@@ -143,8 +133,14 @@ def get_webdriver(browser: Browser, extension=True):
             driver.quit()
 
 
-def set_host(*, driver, host: str, port: str):
-    ep = driver.find_element(By.ID, 'host_id') # TODO rename to 'backend'?
+def set_host(*, driver, host: Optional[str], port: Optional[str]) -> None:
+    # todo rename to 'backend_id'?
+    if host is None:
+        ep = driver.find_element(By.ID, 'host_id')
+        ep.clear()
+        return
+    assert port is not None
+    ep = driver.find_element(By.ID, 'host_id')
     ep.clear()
     ep.send_keys(f'{host}:{port}')
 
@@ -159,7 +155,7 @@ def save_settings(driver):
 LOCALHOST = 'http://localhost'
 
 
-def configure(
+def configure_extension(
         driver,
         *,
         host: Optional[str]=LOCALHOST,
@@ -167,6 +163,7 @@ def configure(
         show_dots: bool=True,
         highlights: Optional[bool]=None,
         blacklist=None,
+        excludelists=None,
         notify_contexts: Optional[bool]=None,
         position: Optional[str]=None,
         verbose_errors: bool=True,
@@ -184,10 +181,7 @@ def configure(
     open_extension_page(driver, page='options_page.html')
     sleep(1) # err, wtf? otherwise not always interacts with the elements correctly
 
-    assert not (host is None) ^ (port is None), (host, port)
-    if host is not None:
-        assert port is not None
-        set_host(driver=driver, host=host, port=port)
+    set_host(driver=driver, host=host, port=port)
 
     # dots = driver.find_element(By.ID, 'dots_id')
     # if dots.is_selected() != show_dots:
@@ -211,13 +205,34 @@ def configure(
         bl.click()
         # ugh, that's hacky. presumably due to using Codemirror?
         bla = driver.switch_to.active_element
+        # ugh. doesn't work, .text always returns 0...
+        # if len(bla.text) > 0:
+        # for some reason bla.clear() isn't working...
+        # results in ElementNotInteractableException: Element <textarea> could not be scrolled into view
+        bla.send_keys(Keys.CONTROL + 'a')
+        bla.send_keys(Keys.BACKSPACE)
         bla.send_keys('\n'.join(blacklist))
 
+    if excludelists is not None:
+        excludelists_json = json.dumps(excludelists)
+
+        bl = driver.find_element(By.ID, 'global_excludelists_ext_id') # .find_element_by_tag_name('textarea')
+        bl.click()
+        # ugh, that's hacky. presumably due to using Codemirror?
+        bla = driver.switch_to.active_element
+        # ugh. doesn't work, .text always returns 0...
+        # if len(bla.text) > 0:
+        # for some reason bla.clear() isn't working...
+        # results in ElementNotInteractableException: Element <textarea> could not be scrolled into view
+        bla.send_keys(Keys.CONTROL + 'a')
+        bla.send_keys(Keys.BACKSPACE)
+        bla.send_keys(excludelists_json)
+
     save_settings(driver)
 
 
 # legacy name
-configure_extension = configure
+configure = configure_extension
 
 
 def get_window_id(driver):
@@ -263,12 +278,36 @@ def send_key(key):
     pyautogui.hotkey(*key)
 
 
-def trigger_hotkey(driver, hotkey):
-    trigger_callback(driver, lambda: send_key(hotkey))
+def is_headless(driver) -> bool:
+    if driver.name == 'firefox':
+        return driver.capabilities.get('moz:headless', False)
+    else:
+        return False
 
 
-def trigger_command(driver, cmd):
-    trigger_hotkey(driver, get_hotkey(driver, cmd))
+# TODO move to common or something
+def trigger_hotkey(driver, hotkey: str) -> None:
+    headless = is_headless(driver)
+    try:
+        trigger_callback(driver, lambda: send_key(hotkey))
+    except Exception as e:
+        if headless:
+            raise RuntimeError("Likely failed because the browser is headless") from e
+        else:
+            raise e
+
+
+def trigger_command(driver, cmd: str) -> None:
+    if is_headless(driver):
+        assert cmd == Command.ACTIVATE, cmd  # will support the rest later..
+        # see background-injector.js
+        driver.execute_script("""
+        var event = document.createEvent('HTMLEvents');
+        event.initEvent('selenium-bridge-activate', true, true);
+        document.dispatchEvent(event);
+        """)
+    else:
+        trigger_hotkey(driver, get_hotkey(driver, cmd))
 
 
 class TestHelper(NamedTuple):
@@ -281,17 +320,35 @@ class TestHelper(NamedTuple):
         from selenium.webdriver.common.action_chains import ActionChains # type: ignore
         ActionChains(self.driver).move_to_element(element).perform()
 
-    def switch_to_sidebar(self):
+    def switch_to_sidebar(self, wait=False):
         self.driver.switch_to.default_content()
-        self.driver.switch_to.frame('promnesia-sidebar')
+
+        frame_id = 'promnesia-frame'
+        frame = None
+        if wait:
+            Wait(self.driver, timeout=5).until(
+                EC.presence_of_element_located((By.ID, frame_id))
+            )
+        self.driver.switch_to.frame(frame_id)
+
+    @contextmanager
+    def sidebar(self, wait=False):
+        try:
+            self.switch_to_sidebar(wait=wait)
+            if wait: # meh
+                yield self.driver.find_element_by_id('promnesia-sidebar')
+            else:
+                yield None
+        finally:
+            self.driver.switch_to.default_content()
 
     def command(self, cmd):
         trigger_command(self.driver, cmd)
 
-    def activate(self):
+    def activate(self) -> None:
         self.command(Command.ACTIVATE)
 
-    def mark_visited(self):
+    def mark_visited(self) -> None:
         self.command(Command.MARK_VISITED)
 
     def wid(self) -> str:
@@ -303,6 +360,12 @@ class TestHelper(NamedTuple):
 
 
 def confirm(what: str) -> None:
+    is_headless = 'headless' in os.environ.get('PYTEST_CURRENT_TEST', '')
+    if is_headless:
+        # ugh.hacky
+        Headless().confirm(what)
+        return
+
     import click # type: ignore
     click.confirm(click.style(what, blink=True, fg='yellow'), abort=True)
 
@@ -407,8 +470,10 @@ def test_settings(tmp_path, browser):
 @browsers()
 def test_backend_status(tmp_path, browser):
     with get_webdriver(browser=browser) as driver:
-        open_extension_page(driver, page='options_page.html')
+        helper = TestHelper(driver)
+        helper.open_page('options_page.html')
         sleep(1) # ugh. for some reason pause here seems necessary..
+
         set_host(driver=driver, host='https://nosuchhost.com', port='1234')
         driver.find_element(By.ID, 'backend_status_id').click()
         sleep(1 + 0.5) # needs enough time for timeout to trigger...
@@ -438,8 +503,7 @@ def set_position(driver, settings: str):
         area.send_keys(settings)
 
 
-# TODO ffh?
-@browsers(FF, CH)
+@browsers()
 def test_sidebar_bottom(browser):
     with get_webdriver(browser=browser) as driver:
         open_extension_page(driver, page='options_page.html')
@@ -464,21 +528,66 @@ def test_sidebar_bottom(browser):
         confirm("You should see sidebar below")
 
 
-@browsers(FF, CH)
-def test_blacklist_custom(tmp_path, browser) -> None:
+@browsers()
+def test_blacklist_custom(tmp_path: Path, browser: Browser) -> None:
     with get_webdriver(browser=browser) as driver:
+        helper = TestHelper(driver)
         configure_extension(driver, port='12345', blacklist=('stackoverflow.com',))
-        driver.get('http://stackoverflow.com')
-        trigger_command(driver, Command.ACTIVATE)
+        driver.get('https://stackoverflow.com/questions/27215462')
+
+        helper.activate()
+
         manual.confirm('page should be blacklisted (black icon), your should see an error notification')
 
+        # make sure we can't open sidebar for blacklisted page
+        # meh, but that'll do for now
+        from selenium.common.exceptions import NoSuchFrameException
+        with pytest.raises(NoSuchFrameException):
+            with helper.sidebar():
+                pass
+
+        # reset blacklist
+        # also running without backend here, so need to set host to none as well
+        configure_extension(driver, host=None, blacklist=())
+        driver.back() # TODO maybe configure_extension should go back instead...
+        driver.refresh()
+
+        with helper.sidebar(wait=True):
+            # at least shouln't fail
+            pass
+
+        manual.confirm('sidebar should be visible')
+
 
 @browsers()
-def test_blacklist_builtin(tmp_path, browser):
+def test_blacklist_builtin(tmp_path: Path, browser: Browser) -> None:
     with get_webdriver(browser=browser) as driver:
+        helper = TestHelper(driver)
         configure_extension(driver, port='12345')
         driver.get('https://www.hsbc.co.uk/mortgages/')
-        manual.confirm('page should be blacklisted (black icon)')
+
+        helper.activate()
+
+        manual.confirm('page should be blacklisted (black icon), your should see an error notification')
+
+        # make sure we can't open sidebar for blacklisted page
+        # meh, but that'll do for now
+        from selenium.common.exceptions import NoSuchFrameException
+        with pytest.raises(NoSuchFrameException):
+            with helper.sidebar():
+                pass
+
+        # reset blacklist
+        # also running without backend here, so need to set host to none as well
+        configure_extension(driver, host=None, excludelists=())
+        driver.back() # TODO maybe configure_extension should go back instead...
+        driver.refresh()
+
+        with helper.sidebar(wait=True):
+            # at least shouln't fail
+            pass
+
+        manual.confirm('sidebar should be visible')
 
 
 @browsers(FF, CH)
@@ -507,13 +616,47 @@ def test_add_to_blacklist(tmp_path, browser):
         confirm('page should be blacklisted (black icon)')
 
 
-@browsers(FF, CH)
-def test_visits(tmp_path, browser):
+
+# todo might be nice to run soft asserts for this test?
+@browsers()
+def test_visits(tmp_path: Path, browser: Browser) -> None:
     test_url = "http://www.e-flux.com/journal/53/59883/the-black-stack/"
     # test_url = "file:///usr/share/doc/python3/html/library/contextlib.html" # TODO ??
     with _test_helper(tmp_path, index_hypothesis, test_url, browser=browser) as helper:
-        trigger_command(helper.driver, Command.ACTIVATE)
-        confirm('you should see hypothesis contexts')
+        driver = helper.driver
+        with helper.sidebar():
+            # hmm not sure how come it returns anything at all.. but whatever
+            srcs = driver.find_elements_by_class_name('src')
+            for s in srcs:
+                assert not s.is_displayed(), s
+            assert len(srcs) >= 8, srcs
+            # todo ugh, need to filter out filters, how to only query the ones in the sidebar?
+
+        helper.activate()
+
+        with helper.sidebar(wait=True) as sidebar:
+            assert sidebar is not None
+
+            confirm('you should see hypothesis contexts')
+
+            link = driver.find_element_by_partial_link_text('how_algorithms_shape_our_world')
+            assert link.is_displayed(), link
+
+            contexts = helper.driver.find_elements_by_class_name('context')
+            for c in contexts:
+                assert c.is_displayed(), c
+            assert len(contexts) == 8
+
+
+        helper.activate()
+        # todo wait till it disappears properly?
+        sleep(0.5)
+
+        with helper.sidebar():
+            # hmm not sure how come it returns anything at all.. but whatever
+            srcs = driver.find_elements_by_class_name('src')
+            for s in srcs:
+                assert not s.is_displayed(), s
 
 
 @browsers()
@@ -552,6 +695,31 @@ def test_show_visited_marks(tmp_path, browser):
         confirm("You should see visited marks near special linear group, Unitary group, Transpose")
 
 
+@browsers(FF, CH)
+@pytest.mark.parametrize(
+    'url',
+    [
+        "https://en.wikipedia.org/wiki/Symplectic_group",
+
+        # regression test for https://github.com/karlicoss/promnesia/issues/295
+        # note: seemed to reproduce on chrome more consistently for some reason
+        "https://www.udemy.com/course/javascript-bible/",
+    ],
+    ids=[
+        'wiki',
+        'udemy'
+    ],
+)
+def test_sidebar(tmp_path, browser, url: str) -> None:
+    visited = {
+        url : 'whatever',
+    }
+    indexer = index_urls(visited, source_name='ælso test unicode 💩')
+    with _test_helper(tmp_path, indexer, url, show_dots=True, browser=browser) as helper:
+        trigger_command(helper.driver, Command.ACTIVATE)
+        confirm("You should see green icon, also one visit in sidebar. Make sure the unicode is displayed correctly.")
+
+
 @browsers(FF, CH)
 def test_search(tmp_path, browser):
     test_url = "https://en.wikipedia.org/wiki/Symplectic_vector_space"
@@ -580,10 +748,8 @@ def test_new_background_tab(tmp_path, browser) -> None:
 
 
 # TODO shit disappears on chrome and present on firefox
-@browsers(FF, CH)
-# FIXME fails on headless firefox because in docker python3-doc isn't installing this file??
-# really weird, mismatches dpkg -L
-def test_local_page(tmp_path, browser) -> None:
+@browsers()
+def test_local_page(tmp_path: Path, browser: Browser) -> None:
     tutorial = 'file:///usr/share/doc/python3/html/tutorial/index.html'
     urls = {
          tutorial                                                : 'TODO read this',
